@@ -16,8 +16,9 @@
 
 typedef enum {
     json_create_ok,
-    /* Unknown Perl svtype within the structure. */
-    json_create_unknown_type,
+
+    /* Something went wrong in our code (we goofed) */
+
     /* An error from the unicode.c library. */
     json_create_unicode_error,
     /* A printed number turned out to be longer than MARGIN bytes. */
@@ -26,10 +27,17 @@ typedef enum {
     json_create_unknown_floating_point,
     /* Bad format for floating point. */
     json_create_bad_floating_format,
+
+    /* User-generated exceptions (bad input) */
+
     /* */
     json_create_unicode_bad_utf8,
+    /* Unknown Perl svtype within the structure. */
+    json_create_unknown_type,
     /* User's routine returned invalid stuff. */
     json_create_invalid_user_json,
+    /* User gave us an undefined value from a user subroutine. */
+    json_create_undefined_return_value,
 }
 json_create_status_t;
 
@@ -116,9 +124,10 @@ static int (* json_create_error_handler) (const char * file, int line_number, co
 	case json_create_unknown_type:				\
 	case json_create_unicode_bad_utf8:			\
 	case json_create_invalid_user_json:			\
+	case json_create_undefined_return_value:		\
 	    break;						\
 	    							\
-	    /* All other exceptions are bugs. */		\
+	    /* All other exceptions are our bugs. */		\
 	default:						\
 	    if (JCEH) {						\
 		(*JCEH) (__FILE__, __LINE__,			\
@@ -379,7 +388,7 @@ json_create_add_key_len (json_create_t * jc, const unsigned char * key, STRLEN k
 
     CALL (add_char (jc, '"'));
     for (i = 0; i < keylen; ) {
-	unsigned char c, d, e;
+	unsigned char c, d, e, f;
 	c = key[i];
 
 	switch (jump[c]) {
@@ -495,14 +504,37 @@ json_create_add_key_len (json_create_t * jc, const unsigned char * key, STRLEN k
 	    break;
 
 	case UT4:
+	    d = key[i + 1];
+	    e = key[i + 2];
+	    f = key[i + 3];
+	    if (
+		// These byte values are copied from
+		// https://github.com/htacg/tidy-html5/blob/768ad46968b43e29167f4d1394a451b8c6f40b7d/src/utf8.c
+
+		// 0x40000 - 0xfffff
+		(c < 0xf4 &&
+		 (d < 0x80 || d > 0xBF ||
+		  e < 0x80 || e > 0xBF ||
+		  f < 0x80 || f > 0xBF))
+		||
+		// 0x100000 - 0x10ffff
+		(c == 0xf4 && 
+		 (d < 0x80 || d > 0x8F ||
+		  e < 0x80 || e > 0xBF ||
+		  f < 0x80 || f > 0xBF))
+		) {
+		BADUTF8;
+		i++;
+		break;
+	    }
 	    if (jc->unicode_escape_all) {
 		unsigned int u;
 		const unsigned char * input;
 		input = key + i;
-		u = (input[0] & 0x07) << 18
-		  | (input[1] & 0x3F) << 12
-		  | (input[2] & 0x3F) <<  6
-		  | (input[3] & 0x3F);
+		u = (c & 0x07) << 18
+		  | (d & 0x3F) << 12
+		  | (e & 0x3F) <<  6
+		  | (f & 0x3F);
 		add_u (jc, u);
 	    }
 	    else {
@@ -708,7 +740,8 @@ json_create_add_float (json_create_t * jc, SV * sv)
 	    fvlen = snprintf ((char *) jc->buffer + jc->length, MARGIN, jc->fformat, fv);
 	}
 	else {
-	    fvlen = snprintf ((char *) jc->buffer + jc->length, MARGIN, "%g", fv);
+	    fvlen = snprintf ((char *) jc->buffer + jc->length, MARGIN,
+			      "%g", fv);
 	}
 	if (fvlen >= MARGIN) {
 	    return json_create_number_too_long;
@@ -882,6 +915,13 @@ json_create_call_to_json (json_create_t * jc, SV * cv, SV * r)
     SvREFCNT_inc (json);
     FREETMPS;
     LEAVE;  
+    if (! SvOK (json)) {
+	/* User returned an undefined value. */
+	SvREFCNT_dec (json);
+	json_create_user_message (jc, json_create_undefined_return_value,
+				  "undefined value from user routine");
+	return json_create_undefined_return_value;
+    }
     jsonc = SvPV (json, jsonl);
     if (jc->validate) {
 	CALL (json_create_validate_user_json (jc, json));
@@ -992,9 +1032,6 @@ json_create_handle_object (json_create_t * jc, SV * input)
 		else if (SvROK (*sv_ptr)) {
 		    SV * what;
 		    what = SvRV (*sv_ptr);
-		    if (SvROK (what)) {
-			what = SvRV(what);
-		    }
 		    switch (SvTYPE (what)) {
 		    case SVt_PVCV:
 			CALL (json_create_call_to_json (jc, what, r));
@@ -1092,14 +1129,18 @@ json_create_recursively (json_create_t * jc, SV * input)
 	    break;
 
 	case SVt_IV:
+//	    fprintf (stderr, "SVt_IV %ld\n", SvIV (r));
 	    CALL (json_create_add_integer (jc, r));
 	    break;
 
 	case SVt_NV:
+//	    fprintf (stderr, "%lu %d %lu %lu\n", SvIOK (r), SvIOK_UV (r), SvNOK (r), SvNIOK (r));
+//	    fprintf (stderr, "SVt_NV %g\n", SvNV (r));
 	    CALL (json_create_add_float (jc, r));
 	    break;
 
 	case SVt_PVNV:
+//	    fprintf (stderr, "SVt_PVNV %g\n", SvNV (r));
 	    /* We need to handle non-finite numbers without using
 	       Perl's stringified forms, because we need to put quotes
 	       around them, whereas Perl will just print 'nan' the
